#include <jni.h>
#include <R.h>
#include <Rdefines.h>
#include <R_ext/Rdynload.h>
#include <R_ext/Print.h>
#include "RWebServicesConverters.h"

/* pointers to c functions in package SJava */
static DL_FUNC sjava_getJNIEnv, sjava_anonymousAssign, sjava_s_to_java_basic;
static DL_FUNC sjava_JavaStringArray, sjava_JavaObjectArray;

void
RWebServices_init(SEXP info_getJNIEnv, SEXP info_JavaStringArray, 
		SEXP info_JavaObjectArray, SEXP info_s_to_java_basic,
		SEXP info_anonymousAssign)
{
  sjava_getJNIEnv = R_ExternalPtrAddr(info_getJNIEnv);
  sjava_JavaStringArray = R_ExternalPtrAddr(info_JavaStringArray);
  sjava_JavaObjectArray = R_ExternalPtrAddr(info_JavaObjectArray);
  sjava_anonymousAssign = R_ExternalPtrAddr(info_anonymousAssign);
  sjava_s_to_java_basic = R_ExternalPtrAddr(info_s_to_java_basic);
}

SEXP
RCharScalar_JavaString( SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH( obj );
  jobject value;
  SEXP ans;

  if ( n > 1 ) {
      REprintf("RCharScalar_JavaString: input is a R character vector of length > 1. Only convert the 1st element to Java String.\n");
  } 
  value = (*env)->NewStringUTF( env, CHAR( STRING_ELT( obj, 0 )));

  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RIntegerScalar_JavaInteger(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  jclass thisClass = (*env)->FindClass(env, "java/lang/Integer");
  jmethodID thisConstructor= (*env)->GetMethodID(env, thisClass, "<init>", "(I)V");
  jobject value;
  SEXP ans;

  value = (*env)->NewObject(env, thisClass, thisConstructor, INTEGER_POINTER(obj)[0]);

  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, thisClass);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RNumericScalar_JavaDouble(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  jclass thisClass = (*env)->FindClass(env, "java/lang/Double");
  jmethodID thisConstructor= (*env)->GetMethodID(env, thisClass, "<init>", "(D)V");
  jobject value;
  SEXP ans;

  value = (*env)->NewObject(env, thisClass, thisConstructor, NUMERIC_POINTER(obj)[0]);

  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, thisClass);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RLogicalScalar_JavaBoolean(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  jclass thisClass = (*env)->FindClass(env, "java/lang/Boolean");
  jmethodID thisConstructor= (*env)->GetMethodID(env, thisClass, "<init>", "(Z)V");
  jobject value;
  SEXP ans;

  value = (*env)->NewObject(env, thisClass, thisConstructor, LOGICAL_POINTER(obj)[0]);

  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, thisClass);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RCharVector_JavaStringArray(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH( obj ), i;
  jstring       str;
  jobject value;
  SEXP ans;

    value = sjava_JavaStringArray(env, n);
    for (i=0; i < n; ++i ) {
      str = (*env)->NewStringUTF( env, CHAR( STRING_ELT( obj, i )));
      (*env)->SetObjectArrayElement(env, value, i, str);
      (*env)->DeleteLocalRef(env, str);
    }

  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RList_JavaObjectArray(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH( obj ), i;
  jobject ele;
  jobject value;
  SEXP ans;

    value = sjava_JavaObjectArray(env, n);
    for (i=0; i < n; ++i ) {
      ele = sjava_s_to_java_basic(env, VECTOR_ELT( obj, i ), NULL, NULL);
      (*env)->SetObjectArrayElement(env, value, i, ele);
      (*env)->DeleteLocalRef(env, ele);
    }

  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RIntegerVector_JavaIntArray(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH(obj), i;
  jintArray value;
  jint *els;
  jboolean isCopy;
  SEXP ans;

  value = (*env)->NewIntArray(env, n);
  els = (*env)->GetIntArrayElements(env, value, &isCopy);
  for(i = 0; i < n ; i++)
     els[i] = INTEGER_POINTER(obj)[i];

  if(isCopy)
     (*env)->ReleaseIntArrayElements(env, value, els, 0);
  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RNumericVector_JavaDoubleArray(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH(obj), i;
  jdoubleArray value;
  jdouble *els;
  jboolean isCopy;
  SEXP ans;

  value = (*env)->NewDoubleArray(env, n);
  els = (*env)->GetDoubleArrayElements(env, value, &isCopy);
  for(i = 0; i < n ; i++)
     els[i] = NUMERIC_POINTER(obj)[i];

  if(isCopy)
     (*env)->ReleaseDoubleArrayElements(env, value, els, 0);
  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RLogicalVector_JavaBooleanArray(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH(obj), i;
  jbooleanArray value;
  jboolean *els;
  jboolean isCopy;
  SEXP ans;

  value = (*env)->NewBooleanArray(env, n);
  els = (*env)->GetBooleanArrayElements(env, value, &isCopy);
  for(i = 0; i < n ; i++)
     els[i] = LOGICAL_POINTER(obj)[i];

  if(isCopy)
     (*env)->ReleaseBooleanArrayElements(env, value, els, 0);
  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

SEXP
RRawVector_JavaByteArray(SEXP obj)
{
  JNIEnv *env = sjava_getJNIEnv();
  int n = GET_LENGTH(obj), i;
  jbyteArray value;
  jbyte *els;
  jboolean isCopy;
  SEXP ans;

  value = (*env)->NewByteArray(env, n);
  els = (*env)->GetByteArrayElements(env, value, &isCopy);
  for(i = 0; i < n ; i++)    {
     els[i] = RAW_POINTER(obj)[i];
  }
  if(isCopy)
     (*env)->ReleaseByteArrayElements(env, value, els, 0);
  ans = sjava_anonymousAssign(env, value, NULL);
  (*env)->DeleteLocalRef(env, value);
  return(ans);
}

/* 
** here is an example of usage
**
###################################################
#    R integer vector to Java rtypes.RInteger 
###################################################
cvtIntegerToJava <- function(x, ...) {
    value <- .Call("RIntegerVector_JavaIntArray", x)
    name <- names(x)
    if (!is.null(name))
        name <- .Call("RCharVector_JavaStringArray", name)
    ans <- .JNew("rtypes.RInteger", .convert=FALSE)
    .Java(ans, "setNames", name, .convert=FALSE)
    .Java(ans, "setValue", value, .convert=FALSE)
    ans
}

matchIntegerToJava <- function(x, ...) {
    is.vector(x) && is.integer(x)
}

setJavaFunctionConverter (cvtIntegerToJava, matchIntegerToJava, 
    description="R integer  to rtypes.RInteger",
    fromJava=FALSE, position=1)
*/
